//+------------------------------------------------------------------+
//|                                             SuperTrendSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "SuperTrend.ex5"
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,   SuperTrend                |
//| Type=SignalAdvanced                                              |
//| Name=SuperTrend                                                  |
//| Class=CSuperTrendSignal                                          |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=CCIPeriod,uint,50,  CCI                |
//| Parameter=ATRPeriod,uint,5,  ATR                 |
//| Parameter=Level,uint,0,  CCI                  |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CSuperTrendSignal.                                         |
//| :            |
//|  SuperTrend http://www.mql5.com/ru/code/527/.          |
//|                 CExpertSignal.        |
//+------------------------------------------------------------------+
class CSuperTrendSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      SuperTrend

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   
   uint              m_CCIPeriod;        //   CCI
   uint              m_ATRPeriod;        //   ATR
   uint              m_Level;            //   CCI 
   uint              m_SignalBar;        //      

public:
                     CSuperTrendSignal();

   //---    
   void               BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void               SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void               BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void               SellPosClose(bool value)                { m_SellPosClose=value;     }
   //---   
   void               Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void               CCIPeriod(uint value)                   { m_CCIPeriod=value;        }
   void               ATRPeriod(uint value)                   { m_ATRPeriod=value;        }
   void               Level(uint value)                       { m_Level=value;            }
   //---
   void               SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              InitSuperTrend(CIndicators *indicators);   // ,   SuperTrend
  };
//+------------------------------------------------------------------+
//|  CSuperTrendSignal.                                   |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CSuperTrendSignal::CSuperTrendSignal()
  {
//---   -
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;

//---      
   m_Ind_Timeframe=PERIOD_H4;
   m_CCIPeriod=50;
   m_ATRPeriod=5;
   m_Level=0;
//---  
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSuperTrendSignal::ValidationSettings()
  {
//---  
   if(m_CCIPeriod<=0)
     {
      printf(__FUNCTION__+":   CCI    ");
      return(false);
     }

   if(m_ATRPeriod<=0)
     {
      printf(__FUNCTION__+":   ATR    ");
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSuperTrendSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitSuperTrend(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   SuperTrend.                             |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSuperTrendSignal::InitSuperTrend(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[5];
   
   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="SuperTrend.ex5";

   parameters[1].type=TYPE_UINT;
   parameters[1].integer_value=m_CCIPeriod;

   parameters[2].type=TYPE_UINT;
   parameters[2].integer_value=m_ATRPeriod;

   parameters[3].type=TYPE_UINT;
   parameters[3].integer_value=m_Level;
   
   parameters[4].type=TYPE_UINT;
   parameters[4].integer_value=m_Level;
//---
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,5,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---  
   if(!m_indicator.NumBuffers(4))  return(false);
   
//---     SuperTrend
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CSuperTrendSignal::LongCondition()
  {
//---       2  SuperTrend
   double Signal=m_indicator.GetData(2,m_SignalBar);

//---    
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_BuyPosOpen)
        {
         if(m_SellPosClose) return(REVERSE_SHORT);
         else return(OPEN_LONG);
        }
      else
        {
         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }

//---      
   if(!m_SellPosClose) return(NO_SIGNAL);

//---      0  SuperTrend
   Signal=m_indicator.GetData(0,m_SignalBar);
   if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_SHORT);

//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CSuperTrendSignal::ShortCondition()
  {
//---       3  SuperTrend
   double Signal=m_indicator.GetData(3,m_SignalBar);

//---    
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_SellPosOpen)
        {
         if(m_BuyPosClose) return(REVERSE_LONG);
         else return(OPEN_SHORT);
        }
      else
        {
         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }

//---      
   if(!m_BuyPosClose) return(NO_SIGNAL);

//---      1  SuperTrend
   Signal=m_indicator.GetData(1,m_SignalBar);
   if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_LONG);

//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
